﻿using System;
using System.Collections.Generic;
using System.Drawing;
using CoreGraphics;
using Foundation;
using UIKit;
using SEEForgeX.Helpers;

namespace SEEForgeX.iOS
{
    // Original Source: http://stackoverflow.com/questions/21029440/xamarin-ios-drawing-onto-image-after-scaling-it
    public class VESLine
    {
        public UIBezierPath Path
        {
            get;
            set;
        }

        public UIColor Color
        {
            get;
            set;
        }

        public byte Index
        {
            get;
            set;
        }
    }


	public class DrawView : UIView
	{
		public DrawView (RectangleF frame) : base (frame)
		{
			DrawPath = new CGPath ();
			CurrentLineColor = UIColor.Black;
			PenWidth = 3.0f;
			Lines = new List<VESLine> ();
		}

		private PointF PreviousPoint;
		private CGPath DrawPath;
		private byte IndexCount;
		private UIBezierPath CurrentPath;
		private List<VESLine> Lines;

		public UIColor CurrentLineColor { get; set; }
		public String ImageFilePath { get; set; }
		public float PenWidth { get; set; }

		private UIImage _image = null;

		public void Clear ()
		{
			Lines.Clear ();
			DrawPath.Dispose ();
			DrawPath = new CGPath ();
			SetNeedsDisplay ();
		}

		public override void TouchesBegan (NSSet touches, UIEvent evt)
		{
			IndexCount++;

			var path = new UIBezierPath
			{
			    LineWidth = PenWidth
			};

		    var touch = (UITouch)touches.AnyObject;
			PreviousPoint = (PointF)touch.PreviousLocationInView (this);

			var newPoint = touch.LocationInView (this);
			path.MoveTo (newPoint);

			InvokeOnMainThread (SetNeedsDisplay);

			CurrentPath = path;

			var line = new VESLine
			{
				Path = CurrentPath, 
				Color = CurrentLineColor,
				Index = IndexCount 
			};

			Lines.Add (line);
		}

		public override void TouchesMoved (NSSet touches, UIEvent evt)
		{
			var touch = (UITouch)touches.AnyObject;
			var currentPoint = touch.LocationInView (this);

			if (Math.Abs (currentPoint.X - PreviousPoint.X) >= 4 ||
			    Math.Abs (currentPoint.Y - PreviousPoint.Y) >= 4) {

                    var newPoint = new PointF((float)(currentPoint.X + PreviousPoint.X) / 2, (float)(currentPoint.Y + PreviousPoint.Y) / 2);

				CurrentPath.AddQuadCurveToPoint (newPoint, PreviousPoint);
				PreviousPoint = (PointF)currentPoint;
			} else {
				CurrentPath.AddLineTo (currentPoint);
			}

			InvokeOnMainThread (SetNeedsDisplay);
		}

		public override void TouchesEnded (NSSet touches, UIEvent evt)
		{
			InvokeOnMainThread (SetNeedsDisplay);
		}

		public override void TouchesCancelled (NSSet touches, UIEvent evt)
		{
			InvokeOnMainThread (SetNeedsDisplay);
		}

        //public override void Draw (RectangleF rect)
        //{
        //    //if (_image != null)
        //    //	_image.Draw (rect);

        //    foreach (var line in Lines) {
        //        line.Color.SetStroke ();
        //        line.Path.Stroke ();
        //    }
        //}

        public override void Draw(CGRect rect)
        {
            foreach (VESLine p in Lines)
            {
                p.Color.SetStroke();
                p.Path.Stroke();
            }
        }

		public void LoadImageFromFile()
		{
			if (ImageFilePath != null && ImageFilePath != "") {
				_image = ImageHelper.GetRotateImage (ImageFilePath);
				//_image = UIImage.FromFile (ImageFilePath);
			}
		}

		public UIImage GetImageFromView()
		{
			RectangleF rect;
			rect = (RectangleF)Frame;

			UIGraphics.BeginImageContext (rect.Size);

			CGContext context = UIGraphics.GetCurrentContext ();
			if (_image != null)
				context.DrawImage (Frame, _image.CGImage);
			this.Layer.RenderInContext (context);

			UIImage image = UIGraphics.GetImageFromCurrentImageContext();

			UIGraphics.EndImageContext ();

			return image;
		}
	}
}

